@@ -13,6 +13,7 @@
 #'   variance-covariance model used to calculate the \code{CR2} and \code{CR4}
 #'   adjustment matrices. If a vector, the target matrix is assumed to be
 #'   diagonal. If not specified, the target is taken to be an identity matrix.
+#' @param inverse_var Not used for \code{ivreg} objects.
 #' @inheritParams vcovCR
 #'   
 #' @return An object of class \code{c("vcovCR","clubSandwich")}, which consists
@@ -20,51 +21,58 @@
 #'   regression coefficient estimates.
 #'   
 #' @seealso \code{\link{vcovCR}}
-#'   
+#'
+#' @examples 
+#' 
+#' 
+#' if (requireNamespace("AER", quietly = TRUE)) withAutoprint({
+#' 
+#'   library(AER)
+#'   data("CigarettesSW")
+#'   Cigs <- within(CigarettesSW, {
+#'     rprice <- price/cpi
+#'     rincome <- income/population/cpi
+#'     tdiff <- (taxs - tax)/cpi
+#'   })
+#' 
+#'   iv_fit <- ivreg(log(packs) ~ log(rprice) + log(rincome) | 
+#'                   log(rincome) + tdiff + I(tax/cpi), data = Cigs)
+#'   vcovCR(iv_fit, cluster = Cigs$state, type = "CR2")
+#'   coef_test(iv_fit, vcov = "CR2", cluster = Cigs$state)
+#'
+#' })
+#' 
 #' @export
 
-vcovCR.ivreg <- function(obj, cluster, type, target = NULL, inverse_var = NULL, form = "sandwich", ...) {
+vcovCR.ivreg <- function(obj, cluster, type, target = NULL, inverse_var = FALSE, form = "sandwich", ...) {
   if (missing(cluster)) stop("You must specify a clustering variable.")
-  if (is.null(inverse_var)) inverse_var <- is.null(weights(obj)) & is.null(target)
+  if (inverse_var != FALSE) stop("Unfortunately, the inverse_var option is not available for ivreg models.")
   vcov_CR(obj, cluster = cluster, type = type, 
           target = target, inverse_var = inverse_var, form = form)
 }
 
 # residuals_CS()
 # coef()
+# targetVariance()
+# weightMatrix()
+# v_scale()
 
 #----------------------------------------------
 # get X matrix
 #----------------------------------------------
 
+#' @export
+
 model_matrix.ivreg <- function(obj) {
-  model.matrix(obj, component = "regressors")
-}
-
-#----------------------------------------------
-# get projection matrix
-#----------------------------------------------
-
-projection_matrix.ivreg <- function(obj) {
-  model.matrix(obj, component = "projected")
-}
-
-#--------------------------------------------
-# Get (model-based) working variance matrix 
-#-------------------------------------------
-
-targetVariance.ivreg <- function(obj) {
-  rep(1, nobs(obj))
-}
-
-#-------------------------------------
-# Get weighting matrix
-#-------------------------------------
-
-weightMatrix.ivreg <- function(obj) {
-  weights <- weights(obj)
-  if (is.null(weights)) weights <- 1
-  rep(weights, length.out = nobs(obj))
+  model_matrix <- model.matrix(obj, component = "projected")
+  
+  w <- obj$weights
+  if (is.null(w) || all(pos_wts <- w > 0)) {
+    return(model_matrix)
+  } else {
+    return(model_matrix[pos_wts > 0,,drop=FALSE])
+  }
+  
 }
 
 #---------------------------------------
@@ -72,7 +80,4 @@ weightMatrix.ivreg <- function(obj) {
 #---------------------------------------
 
 # bread.ivreg() is in AER package
-
-v_scale.ivreg <- function(obj) {
-  obj$nobs
-}
+# use default v_scale()
\ No newline at end of file
