@@ -1,6 +1,9 @@
 context("ivreg objects")
 set.seed(20190513)
 
+skip_if_not_installed("zoo")
+skip_if_not_installed("AER")
+
 library(zoo, quietly=TRUE)
 library(AER, quietly=TRUE)
 
@@ -104,7 +107,7 @@ test_that("vcovCR options don't matter for CR0", {
 test_that("vcovCR options work for CR2", {
   
   CR2_iv <- vcovCR(obj_un, cluster = Cigs$state, type = "CR2")
-  expect_identical(vcovCR(obj_un, cluster = Cigs$state, type = "CR2", target = rep(1, nobs(obj_un))), CR2_iv)
+  expect_equal(vcovCR(obj_un, cluster = Cigs$state, type = "CR2", target = rep(1, nobs(obj_un))), CR2_iv)
   expect_false(identical(vcovCR(obj_un, cluster = Cigs$state, type = "CR2", target = 1 / Cigs$population), CR2_iv))
   
   wCR2_id <- vcovCR(obj_wt, cluster = Cigs$state, type = "CR2")
@@ -167,11 +170,11 @@ test_that("clubSandwich works with dropped observations", {
   
   CR_drop <- lapply(CR_types, function(x) vcovCR(iv_dropped, cluster = dat_miss$state, type = x))
   CR_complete <- lapply(CR_types, function(x) vcovCR(iv_complete, cluster = dat_complete$state, type = x))
-  expect_identical(CR_drop, CR_complete)
+  expect_equal(CR_drop, CR_complete)
   
   test_drop <- lapply(CR_types, function(x) coef_test(iv_dropped, vcov = x, cluster = dat_miss$state, test = "All", p_values = FALSE))
   test_complete <- lapply(CR_types, function(x) coef_test(iv_complete, vcov = x, cluster = dat_complete$state, test = "All", p_values = FALSE))
-  expect_identical(test_drop, test_complete)
+  expect_equal(test_drop, test_complete)
 })
 
 
@@ -197,4 +200,40 @@ test_that("weight scale doesn't matter", {
   
 })
 
+test_that("clubSandwich works with weights of zero.", {
+  
+  n_Cigs <- nrow(Cigs)
+  Cigs$awt <- rpois(n_Cigs, lambda = 1.4)
+  table(Cigs$awt)
+  
+  iv_full <- update(obj_un, weights = awt)
+  Cigs_sub <- subset(Cigs, awt > 0)
+  iv_sub <- update(iv_full, data = Cigs_sub)
+  
+  CR_full <- lapply(CR_types, function(x) vcovCR(iv_full, cluster = Cigs$state, type = x))
+  CR_sub <- lapply(CR_types, function(x) vcovCR(iv_sub, cluster = Cigs_sub$state, type = x))
+  expect_equal(CR_full, CR_sub, check.attributes = FALSE)
+  
+  test_full <- lapply(CR_types, function(x) coef_test(iv_full, vcov = x, cluster = Cigs$state, test = c("z","naive-t","Satterthwaite"), p_values = TRUE))
+  test_sub <- lapply(CR_types, function(x) coef_test(iv_sub, vcov = x, cluster = Cigs_sub$state, test = c("z","naive-t","Satterthwaite"), p_values = TRUE))
+  expect_equal(test_full, test_sub, check.attributes = FALSE)
+  
+  dat_miss <- Cigs
+  miss_indicator <- sample.int(n_Cigs, size = round(n_Cigs / 10))
+  dat_miss$rincome[miss_indicator] <- NA
+  with(dat_miss, table(awt, is.na(rincome)))
+  
+  iv_dropped <- update(iv_full, data = dat_miss)
+  dat_complete <- subset(dat_miss, !is.na(rincome))
+  iv_complete <- update(iv_full, data = dat_complete)
+  
+  CR_drop <- lapply(CR_types, function(x) vcovCR(iv_dropped, cluster = dat_miss$state, type = x))
+  CR_complete <- lapply(CR_types, function(x) vcovCR(iv_complete, cluster = dat_complete$state, type = x))
+  expect_equal(CR_drop, CR_complete)
+  
+  test_drop <- lapply(CR_types, function(x) coef_test(iv_dropped, vcov = x, cluster = dat_miss$state, test = "All", p_values = FALSE))
+  test_complete <- lapply(CR_types, function(x) coef_test(iv_complete, vcov = x, cluster = dat_complete$state, test = "All", p_values = FALSE))
+  expect_equal(test_drop, test_complete)
+  
+})
 
